package eu.ztsh.wymiana.web.controller;

import eu.ztsh.wymiana.exception.InsufficientFundsException;
import eu.ztsh.wymiana.exception.UserNotFoundException;
import eu.ztsh.wymiana.service.CurrencyService;
import eu.ztsh.wymiana.web.model.CurrencyExchangeRequest;
import eu.ztsh.wymiana.web.model.UserResponse;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RequiredArgsConstructor
@Validated
@RestController
@RequestMapping(path = "/api/exchange", produces = "application/json")
public class ExchangeController {

    private final CurrencyService currencyService;

    @Operation(summary = "Perform exchange")
    @ApiResponses(value = {
        @ApiResponse(responseCode = "200",
            description = "Exchange performed successfully",
            content = @Content(mediaType = MediaType.APPLICATION_JSON_VALUE,
                schema = @Schema(implementation = UserResponse.class))),
        @ApiResponse(responseCode = "400",
            description = "Insufficient funds",
            content = @Content(mediaType = MediaType.TEXT_PLAIN_VALUE)),
        @ApiResponse(responseCode = "404",
            description = "User not found",
            content = @Content(mediaType = MediaType.TEXT_PLAIN_VALUE)),
        @ApiResponse(responseCode = "500",
            description = "Another error has occurred",
            content = @Content(mediaType = MediaType.TEXT_PLAIN_VALUE))
    })
    @PostMapping
    public ResponseEntity<Object> exchange(@Valid @RequestBody CurrencyExchangeRequest request) {
        try {
            return ResponseEntity.status(200).body(currencyService.exchange(request));
        } catch (Exception e) {
            var status = switch (e) {
                case InsufficientFundsException ignored -> HttpStatus.BAD_REQUEST;
                case UserNotFoundException ignored -> HttpStatus.NOT_FOUND;
                default -> HttpStatus.INTERNAL_SERVER_ERROR;
            };
            return ResponseEntity.status(status).body(e.getMessage());
        }
    }

}
